<?php

/*
   This program computes temperature
   equivalents on eight scales.

    -------------------
    VALID SCALE SYMBOLS

    SYMBOL     SCALE
    ------  -----------
      F     Fahrenheit
      C     Celsius
      R     Rankine
      K     Kelvin
      Re    R�aumur
      Ro    R�mer
      N     Newton
      De    Delisle
    -------------------

   The N, Re, Ro and De scales are now
   obsolete, but may be encountered in
   very old scientific literature and
   are included here for historical
   reference.

*/

   ob_start();

// Define the program cookie name and set it to expire in 30 days.
// This cookie can be shared by other programs in the same common
// home folder together.

   $CookieName = 'Temperature_Scale_Equivalents';
   $SetToExpireIn30Days = time() + 30*86400;

// Define script path and filename.
   $_AUTHOR_          = 'Jay Tanner';
   $_PROGRAM_VERSION_ = ''; $at = "&#97;&#116;"; $UTC = "&#85;&#84;&#67;";
   $_SCRIPT_PATH_     = Filter_Input(INPUT_SERVER, 'SCRIPT_FILENAME');
   $_RUN_             = Filter_Input(INPUT_POST, 'SCRIPT_NAME');

// Define internal document page title, HTML page heading text and revision date
   $_INTERNAL_TITLE_  = "Temperature Scale Equivalents";
   $_INTERFACE_TITLE_ = "<span style='font-size:150%;'>Temperature Scale Equivalents Calculator</span>";
   $_REVISION_DATE_   = $_PROGRAM_VERSION_ .'Revised: '. GMDate("l, F d, Y $at h:i:s A   $UTC", FileMTime($_SCRIPT_PATH_));

// Define main TextArea text, background
// colors and HTML table row span.
   $TxColor = 'black';
   $BgColor = 'white';
   $ColSpan = 7;




// Do this only if [COMPUTE] button was clicked.
   $w = Filter_Input(INPUT_POST, 'ComputeButton');
   if (!IsSet($w))

// ----------------------------------------------------
// If [COMPUTE] button was clicked and an active cookie
// exists, then restore the previous interface settings
// from it.
     {
      $w = Filter_Input(INPUT_COOKIE, $CookieName);
   if (IsSet($w))
      {
       $CookieDataString = Filter_Input(INPUT_COOKIE, $CookieName);
       list($Arg1) = Preg_Split("[\|]", $CookieDataString);
      }

   else

// Set the initial default interface startup values.

 {
  $Arg1 = '0';


// Store interface settings in cookie.
   $CookieDataString = $Arg1;
   setcookie ($CookieName, $CookieDataString, $SetToExpireIn30Days);
  }
      } // End of  if (!isset(_POST['ComputeButton']))







// =======================================
// Read values of all interface arguments.
// Empty value is set to zero by default.

   $w = Filter_Input(INPUT_POST, 'ComputeButton');
   if (isset($w))
{
   $Arg1 = trim(Filter_Input(INPUT_POST, 'Arg1'));
   if ($Arg1 == '') {$Arg1 = '0';}
   if (is_Numeric($Arg1)) {$Arg1 .= ' F';}


// ------------------------------------
// Store interface arguments in cookie.

   $CookieDataString = "$Arg1";
   SetCookie ($CookieName, $CookieDataString, $SetToExpireIn30Days);
}




// ========================================
// Check for error. FALSE = Error detected.
// Change this code to detect errors.
//
// Valid scale symbols:  F C K R Re Ro N De

   $ErrFlag = TRUE;







// ================================
// Check if error (FALSE) detected.

if ($ErrFlag === FALSE)
 {
  $TxColor = 'white';
  $BgColor = '#CC0000';
  $TextArea2Text = '';

  $TextArea1Text =
"=== ERROR ====================================================================

$ErrMssg";
 }

else

// ===================================================
// Begin computations here if no error detected above.

  {

   $SubAbsZeroText = '';

// Fix spacing of argument, if necessary.
   $Arg1 = UCFirst(Separate_Num_and_Text($Arg1));

   $TempValue = FloatVal($Arg1);
   $TempScale = UCFirst(trim(substr(StrToLower($Arg1), -2)));
   $Arg1 = trim("$TempValue $TempScale");

   if($Arg1 == '0 0') {$Arg1 = '0 F';}

// Compute equivalent on K scale and then convert
// the K value into all the other scales.
// F C K R Re Ro N De

   $F  = Temp_to_Temp ($Arg1, 'F');
   $C  = Temp_to_Temp ($Arg1, 'C');
   $R  = Temp_to_Temp ($Arg1, 'R');
   $K  = Temp_to_Temp ($Arg1, 'K');

   $Re = Temp_to_Temp ($Arg1, 'Re');
   $Ro = Temp_to_Temp ($Arg1, 'Ro');
   $N  = Temp_to_Temp ($Arg1, 'N');
   $De = Temp_to_Temp ($Arg1, 'De');

if ($K < 0) {$BgColor='#CC0000';   $TxColor = 'white';   $SubAbsZeroText = "\n WARNING:\n The given temperature is below absolute zero,\n but the computations were performed anyway.\n\n";
}


   $Equivalents = '';



   $TextArea1Text =
"    TABLE OF TEMPERATURE SCALE EQUIVALENTS
   $SubAbsZeroText
    COMMON SCALES:
    Celsius     $C C
    Fahrehneit  $F F

    ABSOLUTE SCALES:
    Kelvin      $K K
    Rankine     $R R

    OBSOLETE SCALES:
    Newton      $N N
    R&#233;aumur     $Re Re
    R&oslash;mer       $Ro Ro
    Delisle     $De De";
  }

$i = StrPos($TextArea1Text, 'ERROR: Temp_to_Temp()');

if ($i !== FALSE)
{
$TextArea1Text =
"    ERROR:
    Unknown temperature scale symbol.

    -------------------
    VALID SCALE SYMBOLS

    SYMBOL     SCALE
    ------  -----------
      C     Celsius
      F     Fahrenheit
      K     Kelvin
      R     Rankine
      Re    R&#233;aumur
      Ro    R&#248;mer
      N     Newton
      De    Delisle
    -------------------
";

  $TxColor = 'white';
  $BgColor = 'maroon';
}

// Get rid of signed zero value.
  $TextArea1Text = Str_Replace(' +0 ', '  0 ', $TextArea1Text);

// Get rid of all positive signs (+).
  $TextArea1Text = Str_Replace('+', '', $TextArea1Text);


// Determine the number of text rows to use in the output TextAreas.
   $TextArea1Rows = 2 + substr_count($TextArea1Text, "\n");



print <<< _HTML

<!DOCTYPE='HTML'>


<head>

<title>Temperature Scale Equivalents Calculator</title>

<meta name='viewport' content='width=device-width, initial-scale=1'>
<meta http-equiv="content-type" content="text/html; charset=UTF-8">
<meta name="description" content="Temperature Scale Equivalents Calculator">
<meta name="keywords" content="temperature converter, temperature scales, Fahrenheit, Celsius, converter, scale">
<meta name="author" content="Jay Tanner - NeoProgrammics PHPScienceLabs">
<meta http-equiv='pragma'  content='no-cache'>
<meta http-equiv='expires' content='-1'>
<meta name='robots'    content='index,follow'>
<meta name='googlebot' content='index,follow'>
<link rel="canonical" href="https://www.phpsciencelabs.com/metrics-temperature-scale-equivalents-calculator/">

<style type='text/css'>

 BODY
{color:silver; background-color:black;  font-family:Verdana; font-size:100%}

 TABLE
{border:1px solid black;}

 TD
{color:black; background-color:white; line-height:200%; font-size:12px;
padding:6px; text-align:center;}

 DIV
{
 color:black; background-color:white; font-family:Verdana; font-size:110%;
 text-align:justify; padding:4px;
}

 PRE
{color:black; background-color:white; font-family:monospace; font-size:150%;
font-weight:bold; text-align:left; padding:4px;}

 CODE
{font-family:monospace; font-size:150%; font-weight:normal;}

 TEXTAREA
{
 color:black; background:white; font-family:monospace; font-size:150%;
 font-weight:bold; border-radius: 8px; border:1px solid black; padding:4px;
 white-space:pre;
}

 INPUT[type="text"]::-ms-clear {width:0; height:0;}

 INPUT[type="text"]
{
 color:black; background:white; font-size:150%; font-family:monospace;
 font-weight:bold; text-align:center; box-shadow:2px 2px 3px #666666;
 border:2px solid black; border-radius: 4px;
}

 INPUT[type="text"]:focus
{
 background:white; box-shadow:2px 2px 3px #666666; font-family:monospace;
 font-size:150%; border:2px solid blue; text-align:center; font-weight:bold;
}

 INPUT[type="submit"]
{font-weight:bold;}

 INPUT[type="submit"]:hover
{box-shadow:2px 2px 5px #555555; font-weight:bold;}

 HR {background:black; height:2px; border:0px;}

 A:link
{
 font-size:100%; background:transparent; color:blue; font-family:Verdana;
 font-weight:bold; text-decoration:none; line-height:175%; padding:3px;
 border:1px solid transparent;
}

 A:visited {font-size:100%; background:transparent; color:blue;}

 A:hover
{
 font-size:100%; background:yellow; color:black; border:1px solid black;
 box-shadow:1px 1px 3px #222222;
}

 A:active  {font-size:100%; background:yellow; color:black;}

::selection      {background-color:yellow; color:black;}
::-moz-selection {background-color:yellow; color:black;}
</style>

</head>

<body>

<div ID='SmoothScroll' style='background:black;'>
<form name='form1' method='post' action="$_RUN_">

<table width='460' border='0' cellspacing='1' cellpadding='3' align='left'>

<tr><td colspan="$ColSpan" style='color:white; background-color:#000044; border:2px solid white; border-radius:8px 8px 0px 0px;'>$_INTERFACE_TITLE_<br><span>PHP Program by Jay Tanner</span></td></tr>


<tr><td><br><b style='font-size:140%; font-weight:normal;'>Enter Temperature Value and Scale Symbol</b><br>Recognized Scales: <b>&nbsp;C&nbsp;&nbsp;&nbsp;F&nbsp;&nbsp;&nbsp;K&nbsp;&nbsp;&nbsp;R&nbsp;&nbsp;&nbsp;N&nbsp;&nbsp;&nbsp;Re&nbsp;&nbsp;&nbsp;Ro&nbsp;&nbsp;&nbsp;&nbsp;De</b><br>
<input name='Arg1'  type='text' value="$Arg1"  size='24' maxlength='22'>
</td></tr>



<tr><td colspan="$ColSpan" style='background-color:black;'>
<input name='ComputeButton' type='submit' value=' C O M P U T E '>
</td></tr>



<tr>
<td colspan="$ColSpan" style='color:silver; background-color:black;'>
Double-Click Within Text Area to Select ALL Text<br>
<textarea name='TextArea1' ReadOnly cols="46" rows="$TextArea1Rows" OnDblClick='this.select();' OnMouseUp='return true;' style='background:$BgColor; color:$TxColor'>
$TextArea1Text
</textarea></td>
</tr>



<tr>
<td colspan="$ColSpan" style='background:black;'>
<textarea name='TextArea2' cols="51" rows="75" style='color:black; background:LightYellow; font-size:175%; padding:6px;' ReadOnly>
         FORMULAS USED WITHIN THE PROGRAM

     Eight Temperature Scales Are Recognized

                -------------------
                SYMBOL     SCALE
                ------  -----------
                  F     Fahrenheit
                  C     Celsius
                  R     Rankine
                  K     Kelvin
                  Re    R&#233;aumur
                  Ro    R&oslash;mer
                  N     Newton
                  De    Delisle

-------------------------------------------------
STEP 1:
Compute K-scale Equivalent to Input Temperature

The scale symbols below represent the numerical
values on those scales in the formulas.

Given any temperature on one of the eight scales,
to compute its K-scale equivalent:

K = C + 273.15

K = (F - 32)*5/9 + 273.15

K = K

K = (R - 491.67)*5/9 + 273.15

K = Re*5/4 + 273.15

K = (Ro - 7.5)*40/21 + 273.15

K = N*100/33 + 273.15

K = 373.15 - De*2/3

-------------------------------------------------
STEP 2:
Compute table of equivalents on all eight scales,
based on the K-scale value, using the formulas
given below to compute the tabulated values.

C  = K - 273.15

F  = (K - 273.15)*9/5 + 32

K  = K

R  = (K - 273.15)*9/5 + 491.67

Re = (K - 273.15)*4/5

Ro = (K - 273.15)*21/40 + 7.5

N  = (K - 273.15)*33/100

De = (373.15 - K)*3/2

-------------------------------------------------
CAVEAT:

A warning will be displayed in RED text if the
input temperature equates to any value below
absolute zero (0.0 K or 0.0 R).  There are no
negative temperatures on the absolute scales,
by definition, but the computations will still
be performed anyway.
</textarea>
</td>
</tr>


<tr><td><a href='https://en.wikipedia.org/wiki/Conversion_of_scales_of_temperature' target='_blank' title=' Tries to Open in a New Tab '>Temperature Scale Conversions Wiki-Ref</a></td></tr>

<tr><td colspan="$ColSpan" style='color:gray; background:black;'>Program by $_AUTHOR_<br>$_REVISION_DATE_</td></tr>

</table>

</form>
</div>
<br><br><br><br><br><br><br><br><br><br><br>

<script src='Smooth_Scroll.js'></script>
<script>SmoothScroll.init({cancelOnTouch: true});</script>

</body>


_HTML;




/*
   ===========================================================================
   This function interconverts between 8 different temperature scales.

   The R�aumur, R�mer, Newton and Delisle scales may be useful with older
   historical scientific documents where those scales may be encountered.

   ------------------
   SYMBOL     SCALE
   ------  -----------
     F     Fahrenheit
     C     Celsius
     R     Rankine
     K     Kelvin
     Re    R&#233;aumur
     Ro    R&oslash;mer
     N     Newton
     De    Delisle


   ERRORS:
   An error message is returned for invalid arguments and scale symbols.

   ===========================================================================
*/

   function Temp_to_Temp ($FromTempAndScaleStr, $ToScaleStr)
{
   $TScaleSymbols = ' F C K R Re Ro N De ';

   $FTaS = StrToUpper(trim($FromTempAndScaleStr));

// -------------------------------
// Get and check FromScale symbol.
   $FromScale = substr($FTaS, -2);

   if  ($FromScale == 'RE' or $FromScale == 'RO' or $FromScale == 'DE')
       {
        if ($FromScale == 'RE') {$FromScale = 'Re';}
        if ($FromScale == 'RO') {$FromScale = 'Ro';}
        if ($FromScale == 'DE') {$FromScale = 'De';}
       }
   else
       {$FromScale = substr($FTaS, -1);}

   if (StrPos($TScaleSymbols, " $FromScale ") === FALSE)
  {
   return "ERROR: Temp_to_Temp()\n\nInvalid or missing input scale.  Valid scales: $TScaleSymbols";
  }

// -----------------------------
// Get and check ToScale symbol.
   $ToScale = UCFirst(StrToLower(substr($ToScaleStr, -2)));

   if ($ToScale <> 'Re' or $ToScale <> 'Ro' or $ToScale <> 'De')
      {$ToScale = substr($ToScale, -2);}

   if (StrPos($TScaleSymbols, " $ToScale ") === FALSE)
  {
   return "ERROR: Temp_to_Temp()\n\nInvalid or missing output scale.  Valid scales: $TScaleSymbols";
  }

// -----------------------------------------
// Get numerical value of input temperature.
   $T = FloatVal($FTaS);

// ------------------------------------------------
// Convert input temperature into K scale value for
// internal work, depending on FromScale symbol.
   switch ($FromScale)
  {
   case 'F'  : $K = ($T - 32)*5/9 + 273.15;      break;
   case 'C'  : $K = $T + 273.15;                 break;
   case 'K'  : $K = $T;                          break;
   case 'R'  : $K = ($T - 491.67)*5/9 + 273.15;  break;
   case 'Re' : $K = $T*5/4 + 273.15;             break;
   case 'Ro' : $K = ($T - 7.5)*40/21 + 273.15;   break;
   case 'N'  : $K = $T*100/33 + 273.15;          break;
   case 'De' : $K = 373.15 - $T*2/3;             break;
   default   :                                   break;
  }

// ----------------------------------------------------
// Convert K scale value into ToScale value for output.
   switch ($ToScale)
  {
   case 'F'  : $T = ($K - 273.15)*9/5 + 32;      break;
   case 'C'  : $T = $K - 273.15;                 break;
   case 'K'  : $T = $K;                          break;
   case 'R'  : $T = ($K - 273.15)*9/5 + 491.67;  break;
   case 'Re' : $T = ($K - 273.15)*4/5;           break;
   case 'Ro' : $T = ($K - 273.15)*21/40 + 7.5;   break;
   case 'N'  : $T = ($K - 273.15)*33/100;        break;
   case 'De' : $T = (373.15 - $K)*3/2;           break;
   default   :                                   break;
  }

// Format output value to up to 10 decimals.
   $T = Rtrim(Rtrim(SPrintF("%+18.10f", $T),'0'), '.');
   if ($ToScale == 'K' or $ToScale == 'R') {$T = Str_Replace('+', '', $T);}

   return trim($T);

} // end of  Temp_to_Temp(...)





/*
   Valid scale symbols: F C K R Re Ro N De

   Max = 2 characters (Case INsensitive).

   Returns empty string if no valid symbol found.


   function Check_Temp_Scale ($TempScaleStr)
{
   $ValidSymbols = ' F C K R N RE RO DE ';

   $w = trim($TempScaleStr);

   $ScaleSymbol = StrToUpper(substr(trim($w), -2));

   $i = StrPos(' '.$ScaleSymbol.' ', $ValidSymbols);
        if ($i !== FALSE) {return $TempScale;}

}
*/


/*
   ===========================================================================
   This function separates a numerical value followed by a text label by in-
   serting a single space between them, if there is none already, in order to
   facilitate subsequent parsing of the separate string elements.

   Examples:
   The argument ('12.345km')
   would return '12.345 km'

   The argument ('1.2345e-23m')
   would return '1.2345e-23 m'
   etc.

   Any subsequent spaces within the text or label part remain unchanged.
   Example: The argument ('3blackbirds       in   a     cage')
            would return  '3 blackbirds       in   a     cage'
            with only 1 space inserted after the initial number (3) and all
            subsequent spaces considered part of the text label or symbol
            part of the string and preserved.

   If the argument is a purely numerical value, then it is returned as-is.

   If the argument is only a non-numerical string, then it is returned as-is.

   NO DEPENDENCIES
   ===========================================================================
*/

   function Separate_Num_and_Text ($NumAndTextStr)
{
   $X = trim($NumAndTextStr);
   $L = StrLen($X);
   $num = $alp = '';

   for($i=0;   $i < $L;   $i++)
  {
   $x = substr($X,$i,1);

   if (
       Is_Numeric($x)
       or $x == '.'
       or $x == '-'
       or $x == '+'
       or StrToUpper($x) == 'E'
      )
      {$num .= $x;}
   else
      {$alp = trim(substr($X,$i,$L)); break;}
  }
   return trim("$num $alp");
}

// ---------------------------------------------------------------------------






?>


